<?php

namespace Drupal\domain_role_access\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\domain_role_access\Form\DomainRolesForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class DomainRolesController.
 *
 * @package Drupal\domain_role_access\Controller
 */
class DomainRolesController extends ControllerBase {

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Constructs a DomainRolesController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Return the domain entity given the domain ID.
   *
   * @param string $domain_id
   *   Domain ID of the domain to be retrieved.
   *
   * @return \Drupal\domain\Entity\Domain
   *   Domain object.
   *
   * @throws NotFoundHttpException
   *   When no domain is found with the domain ID provided.
   */
  protected function getDomainEntity(string $domain_id) {
    if (!$domain_entity = $this->entityTypeManager->getStorage('domain')->load($domain_id)) {
      throw new NotFoundHttpException();
    }
    /** @var \Drupal\domain\Entity\Domain $domain_entity */
    return $domain_entity;
  }

  /**
   * Page title callback.
   *
   * @return string
   *   Page's title
   */
  public function getTitle($domain) {
    return $this->getDomainEntity($domain)->label() . " roles";
  }

  /**
   * Edit domain roles.
   *
   * @return array
   *   Edit form page.
   */
  public function edit($domain) {
    $domain_entity = $this->getDomainEntity($domain);

    /** @phpstan-ignore-next-line */
    return  $this->formBuilder()->getForm(DomainRolesForm::class, $domain_entity);
  }
}
